import 'package:flutter/material.dart';

//定义扩展方便获取到颜色值
extension ThemeDataExtension on ThemeData {
  AppColorsTheme get appColors => extension<AppColorsTheme>()!;
}

extension AppColorsThemeExtensions on BuildContext {
  AppColorsTheme get appColors => Theme.of(this).extension<AppColorsTheme>()!;
}

class AppColorsTheme extends ThemeExtension<AppColorsTheme> {
  //亮色主题的自定义颜色值
  static const _colorPrimary = Color(0xFF4161D0); //主题色
  static const _colorFCFCFC = Color(0xFFFCFCFC);
  static const _color666666 = Color(0xFF666666);
  static const _colorD7DBE7 = Color(0xffD7DBE7);
  static const _colorBDBDBD = Color(0xFFBDBDBD);
  static const _colorF2F2F2 = Color(0xFFF2F2F2);
  static const _colorFE6C00 = Color(0xFFFE6C00);
  static const _colorD3D3D3 = Color(0xFFD3D3D3);
  static const _color333333 = Color(0xFF333333);
  static const _colorF3F3F3 = Color(0xFFF3F3F3);
  static const _color999999 = Color(0xFF999999);
  static const _colorF2F3F6 = Color(0xFFF2F3F6);

  //暗色主题的一些自定义颜色值
  static const _darkBlackBg = Color(0xFF0F0F0F);
  static const _darkBlackItem = Color(0xFF1D1D1E);
  static const _darkBlackItemLight = Color(0xFF3B3933);
  static const _darkBlackItemLightMost = Color(0xFF5C5850);
  static const _darkBlackItemDivider = Color(0xFF3B3B3F);

  // 页面中真正使用到的颜色名称
  final Color backgroundDefault; //页面背景颜色（偏白）
  final Color backgroundDark; //页面背景颜色(蓝灰蓝)
  final Color btnBgDefault; //按钮背景颜色
  final Color searchFiledBorder; //搜索框的边框颜色
  final Color authFiledHint; //输入框默认的提示文本颜色
  final Color authFiledText; //输入框默认的文本颜色
  final Color authFiledBG; //输入框默认的背景颜色
  final Color textPrimary; //主题色文本
  final Color textBlack; //黑色文本
  final Color textDarkGray; //深灰色 666 文本
  final Color orangeBG; //按钮的橙色背景
  final Color tabBgSelectedPrimary; //Tab的选中主题色背景
  final Color tabBgUnSelectedPrimary; //Tab的未选中主题色背景
  final Color tabTextSelectedPrimary; //Tab的未选中主题色文本
  final Color tabTextUnSelectedPrimary; //Tab的未选中主题色文本
  final Color imgGrayBg; //灰色的图片底色背景
  final Color textDarkGray999; //文本深灰色
  final Color whiteBG; //按钮的白色色背景
  final Color whiteSecondBG; //按钮的白色色背景

  // 私有的构造函数
  const AppColorsTheme._internal({
    required this.backgroundDefault,
    required this.backgroundDark,
    required this.btnBgDefault,
    required this.searchFiledBorder,
    required this.authFiledHint,
    required this.authFiledText,
    required this.authFiledBG,
    required this.textPrimary,
    required this.textBlack,
    required this.textDarkGray,
    required this.orangeBG,
    required this.tabBgSelectedPrimary,
    required this.tabBgUnSelectedPrimary,
    required this.tabTextSelectedPrimary,
    required this.tabTextUnSelectedPrimary,
    required this.imgGrayBg,
    required this.textDarkGray999,
    required this.whiteBG,
    required this.whiteSecondBG,
  });

  // 浅色主题工厂方法
  factory AppColorsTheme.light() {
    return const AppColorsTheme._internal(
      backgroundDefault: _colorFCFCFC,
      backgroundDark: _colorF2F3F6,
      btnBgDefault: _colorPrimary,
      searchFiledBorder: _colorD7DBE7,
      authFiledHint: _colorBDBDBD,
      authFiledText: Colors.black,
      authFiledBG: _colorF2F2F2,
      textPrimary: _colorPrimary,
      textBlack: Colors.black,
      textDarkGray: _color666666,
      orangeBG: _colorFE6C00,
      tabBgSelectedPrimary: _colorPrimary,
      tabBgUnSelectedPrimary: _colorD3D3D3,
      tabTextSelectedPrimary: Colors.white,
      tabTextUnSelectedPrimary: _color333333,
      imgGrayBg: _colorF3F3F3,
      textDarkGray999: _color999999,
      whiteBG: Colors.white,
      whiteSecondBG: Colors.white,
    );
  }

  // 暗色主题工厂方法
  factory AppColorsTheme.dark() {
    return const AppColorsTheme._internal(
      backgroundDefault: _darkBlackBg,
      backgroundDark: _darkBlackBg,
      btnBgDefault: _darkBlackItem,
      searchFiledBorder: _darkBlackItem,
      authFiledHint: _colorD7DBE7,
      authFiledText: Colors.white,
      authFiledBG: _darkBlackItem,
      textPrimary: Colors.white,
      textBlack: Colors.white,
      textDarkGray: Colors.white,
      orangeBG: _darkBlackItem,
      tabBgSelectedPrimary: Colors.white,
      tabBgUnSelectedPrimary: _darkBlackItem,
      tabTextSelectedPrimary: Colors.black,
      tabTextUnSelectedPrimary: Colors.white,
      imgGrayBg: _darkBlackItem,
      textDarkGray999: Colors.white,
      whiteBG: _darkBlackItem,
      whiteSecondBG: _darkBlackItemLight,
    );
  }

  @override
  ThemeExtension<AppColorsTheme> copyWith({bool? lightMode}) {
    if (lightMode == null || lightMode == true) {
      return AppColorsTheme.light();
    }
    return AppColorsTheme.dark();
  }

  @override
  AppColorsTheme lerp(covariant ThemeExtension<AppColorsTheme>? other, double t) {
    if (other is! AppColorsTheme) return this;
    return AppColorsTheme._internal(
      backgroundDefault: Color.lerp(backgroundDefault, other.backgroundDefault, t)!,
      backgroundDark: Color.lerp(backgroundDark, other.backgroundDark, t)!,
      btnBgDefault: Color.lerp(btnBgDefault, other.btnBgDefault, t)!,
      searchFiledBorder: Color.lerp(searchFiledBorder, other.searchFiledBorder, t)!,
      authFiledHint: Color.lerp(authFiledHint, other.authFiledHint, t)!,
      authFiledText: Color.lerp(authFiledText, other.authFiledText, t)!,
      authFiledBG: Color.lerp(authFiledBG, other.authFiledBG, t)!,
      textPrimary: Color.lerp(textPrimary, other.textPrimary, t)!,
      textBlack: Color.lerp(textBlack, other.textBlack, t)!,
      textDarkGray: Color.lerp(textDarkGray, other.textDarkGray, t)!,
      orangeBG: Color.lerp(orangeBG, other.orangeBG, t)!,
      tabBgSelectedPrimary: Color.lerp(tabBgSelectedPrimary, other.tabBgSelectedPrimary, t)!,
      tabBgUnSelectedPrimary: Color.lerp(tabBgUnSelectedPrimary, other.tabBgUnSelectedPrimary, t)!,
      tabTextSelectedPrimary: Color.lerp(tabTextSelectedPrimary, other.tabTextSelectedPrimary, t)!,
      tabTextUnSelectedPrimary: Color.lerp(tabTextUnSelectedPrimary, other.tabTextUnSelectedPrimary, t)!,
      imgGrayBg: Color.lerp(imgGrayBg, other.imgGrayBg, t)!,
      textDarkGray999: Color.lerp(textDarkGray999, other.textDarkGray999, t)!,
      whiteBG: Color.lerp(whiteBG, other.whiteBG, t)!,
      whiteSecondBG: Color.lerp(whiteSecondBG, other.whiteSecondBG, t)!,
    );
  }
}
